#include <ctype.h>
#include <gai/gai.h>
#include "applet.h"

static int default_update = 3000;
static int min_update = 100;
static int max_update = 10000;
static int default_step = 100;
static GdkPixbuf *letter_image[40], *warn_image[40];
static SensorApplet *ai;

static char *about_text = 
"Name: " APPLET_NAME "\n"
"Version: " APPLET_VERSION "\n"
"License: GNU GPL\n"
"Author: Osamu Kayasono <jacobi@jcom.home.ne.jp>\n"
"Summary: sensor_applet\n"
"%description read hardware sensors (lm_sensors) statistics.\n";

static GaiSpinButton sensors_freq = {"Update frequency in ms: ",&default_update,
				     &min_update, &max_update,&default_step, &default_update};


char *label1, *label2, *label3, *label4, *label5;
static GaiTextEntry txt_label1 = {"Sensor 1 [Label]: ", &label1, &label1};
static GaiTextEntry txt_label2 = {"Sensor 2 [Label]: ", &label2, &label2};
static GaiTextEntry txt_label3 = {"Sensor 3 [Label]: ", &label3, &label3};
static GaiTextEntry txt_label4 = {"Sensor 4 [Label]: ", &label4, &label4};
static GaiTextEntry txt_label5 = {"Sensor 5 [Label]: ", &label5, &label5};
char *sensor1, *sensor2, *sensor3, *sensor4, *sensor5;
static GaiTextEntry txt_sensor1 = {"Sensor 1 [Sensor]: ", &sensor1, &sensor1};
static GaiTextEntry txt_sensor2 = {"Sensor 2 [Sensor]: ", &sensor2, &sensor2};
static GaiTextEntry txt_sensor3 = {"Sensor 3 [Sensor]: ", &sensor3, &sensor3};
static GaiTextEntry txt_sensor4 = {"Sensor 4 [Sensor]: ", &sensor4, &sensor4};
static GaiTextEntry txt_sensor5 = {"Sensor 5 [Sensor]: ", &sensor5, &sensor5};

static GaiBox sensors_box[] = {{GAI_SPINBUTTON, (void *)&sensors_freq},
			       {GAI_TEXTENTRY, (void *)&txt_label1},
			       {GAI_TEXTENTRY, (void *)&txt_sensor1},
			       {GAI_TEXTENTRY, (void *)&txt_label2},
			       {GAI_TEXTENTRY, (void *)&txt_sensor2},
			       {GAI_TEXTENTRY, (void *)&txt_label3},
			       {GAI_TEXTENTRY, (void *)&txt_sensor3},
			       {GAI_TEXTENTRY, (void *)&txt_label4},
			       {GAI_TEXTENTRY, (void *)&txt_sensor4},
			       {GAI_TEXTENTRY, (void *)&txt_label5},
			       {GAI_TEXTENTRY, (void *)&txt_sensor5},
			       {GAI_END, NULL}};

static GaiNoteBook sensors_gn[] = {{"sensor_applet - Preferences", (void *)&sensors_box},
				   {NULL,NULL}};

static void load_properties( void )
{
  /* Load saved settings - Default is not to include. */
  default_update = gai_load_int_with_default(APPLET_NAME "/update",3000);
  label1 = gai_load_string_with_default(APPLET_NAME "/label1","CPU");
  label2 = gai_load_string_with_default(APPLET_NAME "/label2","M/B");
  label3 = gai_load_string_with_default(APPLET_NAME "/label3","FAN1");
  label4 = gai_load_string_with_default(APPLET_NAME "/label4","FAN2");
  label5 = gai_load_string_with_default(APPLET_NAME "/label5","CORE");
  sensor1= gai_load_string_with_default(APPLET_NAME "/sensor1","temp2");
  sensor2= gai_load_string_with_default(APPLET_NAME "/sensor2","temp1");
  sensor3= gai_load_string_with_default(APPLET_NAME "/sensor3","fan1");
  sensor4= gai_load_string_with_default(APPLET_NAME "/sensor4","fan2");
  sensor5= gai_load_string_with_default(APPLET_NAME "/sensor5","in0");
}

static void save_properties( void )
{
  /* Load saved settings - Default is not to include. */
  gai_save_int(APPLET_NAME "/update",default_update);
  gai_save_string(APPLET_NAME "/label1", label1);
  gai_save_string(APPLET_NAME "/label2", label2);
  gai_save_string(APPLET_NAME "/label3", label3);
  gai_save_string(APPLET_NAME "/label4", label4);
  gai_save_string(APPLET_NAME "/label5", label5);
  gai_save_string(APPLET_NAME "/sensor1", sensor1);
  gai_save_string(APPLET_NAME "/sensor2", sensor2);
  gai_save_string(APPLET_NAME "/sensor3", sensor3);
  gai_save_string(APPLET_NAME "/sensor4", sensor4);
  gai_save_string(APPLET_NAME "/sensor5", sensor5);

}

void restart_sensors(void)
{
  save_properties();
  gai_signal_on_update_interval_change(default_update);

}

static void readPixmaps( void )
{
  int i;

  for(i = 0; i <= 39; i++) { 
    letter_image[i] = gdk_pixbuf_new_from_xpm_data((const char**)letter_xpm[i]);
    letter_xpm[i][3]="B	c red";
    warn_image[i] = gdk_pixbuf_new_from_xpm_data((const char**)letter_xpm[i]);
  }
}

static gint read_sensors(SensorApplet *ai)
{
  gint i;

  for (i=1;i<=5;i++) {
    memmove(&ai->value[i][1], &ai->value[i][0], sizeof(float)*51); 
    switch (i) {
    case 1:
      ai->value[i][0] = read_val(ai->chips,sensor1,0);
      ai->min[i] = read_val(ai->chips,sensor1,-1);
      ai->max[i] = read_val(ai->chips,sensor1,1);
      break;
    case 2:
      ai->value[i][0] = read_val(ai->chips,sensor2,0);
      ai->min[i] = read_val(ai->chips,sensor2,-1);
      ai->max[i] = read_val(ai->chips,sensor2,1);
      break;
    case 3:
      ai->value[i][0] = read_val(ai->chips,sensor3,0);
      ai->min[i] = read_val(ai->chips,sensor3,-1);
      ai->max[i] = read_val(ai->chips,sensor3,1);
      break;
    case 4:
      ai->value[i][0] = read_val(ai->chips,sensor4,0);
      ai->min[i] = read_val(ai->chips,sensor4,-1);
      ai->max[i] = read_val(ai->chips,sensor4,1);
      break;
    case 5:
      ai->value[i][0] = read_val(ai->chips,sensor5,0);
      ai->min[i] = read_val(ai->chips,sensor5,-1);
      ai->max[i] = read_val(ai->chips,sensor5,1);
      break;
    }
/*  ai->value[i][0] = read_val(ai->chips,ai->sensor[i],0);
    ai->min[i] = read_val(ai->chips,ai->sensor[i],-1);
    ai->max[i] = read_val(ai->chips,ai->sensor[i],1); */
  }
  return (0);
}

static gboolean check_warn(gint i, gchar *path)
{
  /* Fan parameters are current value and low limit */
  if (strncmp(path,"fan",3) == 0) {
      if (ai->value[i][0] !=0 && ai->value[i][0] != DISABLE && 
	  ai->value[i][0] < ai->min[i]) return TRUE;
  }

  /* Temp. parameters are current value and high limit */
  if (strncmp(path,"temp",4) == 0) {
      if (ai->value[i][0] !=0 && ai->value[i][0] != DISABLE && 
	  ai->value[i][0] > ai->max[i]) return TRUE;
  }

  /* Voltages */
  if (ai->value[i][0] !=0 && ai->value[i][0] != DISABLE) {
    if (ai->value[i][0] < ai->min[i] && ai->min[i] != DISABLE) return TRUE;
    if (ai->value[i][0] > ai->max[i] && ai->max[i] != DISABLE) return TRUE;
  }
  return FALSE;
}

static void update_val( void )
{
  gchar msg[10];
  gchar letters[]="+-/.0123456789 ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  gchar *letter;
  gint i,j;
  gboolean warn;
  char label[5], sensor[5];
  /*  read_proc(chips); */
  read_sensors(ai);

  for (i = 1; i <= 5; i++) {
    warn=FALSE;

    label[4]='\0';
    sensor[4]='\0';
    switch (i) {
    case 1:
      strncpy(label, label1, 4);
      strncpy(sensor, sensor1, 4);
      break;
    case 2:
      strncpy(label, label2, 4);
      strncpy(sensor, sensor2, 4);
      break;
    case 3:
      strncpy(label, label3, 4);
      strncpy(sensor, sensor3, 4);
      break;
    case 4:
      strncpy(label, label4, 4);
      strncpy(sensor, sensor4, 4);
      break;
    case 5:
      strncpy(label, label5, 4);
      strncpy(sensor, sensor5, 4);
      break;
    }
    /*    if (strlen(label) > 4) break; */
    if (ai->value[i][0] >= 100) {
      sprintf(msg,"%-4s %4d",label,(int)ai->value[i][0]);
    } else {
      if (ai->value[i][0] >= 10) {
	sprintf(msg,"%-4s %2.1f",label,ai->value[i][0]);
      } else {
	if ((ai->value[i][0] == 0) || (ai->value[i][0] == DISABLE)) {
	  sprintf(msg,"%-4s ----",label);
	} else {
	  sprintf(msg,"%-4s %2.2f",label,ai->value[i][0]);
	}
      }
    }
    warn = check_warn(i,sensor);

    for (j=0;j<strlen(msg);j++) {
      letter=strchr(letters,toupper(msg[j]));
      if(letter != NULL) {
	gai_draw_bg(letter_image[letter-letters], 0,0,6,7,5+j*6,2+i*9);
      } else {
	gai_draw_bg(warn_image[letter-letters], 0,0,6,7,5+j*6,2+i*9);
      }
    }
  }
  gai_draw_update_bg();

}

int main(int argc, char **argv)
{

   int i,j;
   char chips[STRMAX];

   if (find_chips(chips) == 1) {
     printf( "%s : lm_sensors is not activate\n", APPLET_NAME);
     /*      return (1); */
   }

   ai = g_malloc(sizeof(SensorApplet));
   ai->chips = chips;
   for (i = 1; i <= 5; i++) {
      for (j = 0;  j< 52; j++) {
	 ai->value[i][j] = 0;
      }
   }

   /* Initialized the library */
    gai_init(APPLET_NAME, APPLET_VERSION, NULL, &argc, &argv);

    readPixmaps();

    /* Set up text for the about box */
    gai_about_from(about_text);

    /* Set up preference window */
    gai_preferences("sensor_applet preferences", 
		    sensors_gn, 
		    NULL,
		    (GaiCallback0 *)restart_sensors, NULL);

    load_properties();
    gai_signal_on_update((GaiCallback0 *)update_val, default_update, NULL);

    gai_start();
    return 0;

}
